#!/usr/bin/env Rscript
## ---------------------------
##
## Script name: aggregate SPEI spatially
##
## Purpose of script: Aggregate SPEI to spatial regions
## Author: Daniela Lührsen & Emily Ball
## Date Created: 2025-03-07
## Email: daniela.luhrsen@bsc.es
##
## ---------------------------

## load up the packages
packages <- c("startR", "s2dv", "CSTools", "multiApply", "ClimProjDiags",
              "SPEI", "zoo", "TLMoments", "lmomco", "lmom", "lubridate", "dplyr",
              "raster", "sf")
install.packages(setdiff(packages, rownames(installed.packages())))
lapply(packages, require, character.only = TRUE)

## ---------------------------
# select parameters:
region <- "europe"
levels <- c("european_region", "europe", "NUTS3", "ISO3")
accum <- c(6, 12)
start_year <- 1980
end_year <- 2024
start_month <- 1
end_month <- 12

# load SPEI raster
spei6_raster <- readRDS(paste0("../02_calculate_spei/data/", region, "_",
                              "spei_6_stack.rds"))
spei12_raster <- readRDS(paste0("../02_calculate_spei/data/", region, "_",
                              "spei_12_stack.rds"))

# load population raster
pop_density <- readRDS(paste0("../02_calculate_spei/data/", region,
                               "_population_stack.rds"))
cell_areas <- area(pop_density)
pop_era5land <- pop_density * cell_areas

for (level in levels) {
  # load shapefile
  adm1 <- sf::st_read(paste0("../01_shapefiles/data/outputs/", level, ".shp"))
  adm1_id <- adm1$adm1_id
  adm1_name <- adm1$adm1_name

  ntimestep <- raster::nlayers(spei6_raster)
  monthly_adm1 <- data.frame()
  for (l in 1:ntimestep){
    name <- names(spei6_raster[[l]])
    name <- substr(name, 2, 11)
    spei6 <- exactextractr::exact_extract(spei6_raster[[l]], adm1, "mean")
    spei12 <- exactextractr::exact_extract(spei12_raster[[l]], adm1, "mean")
    if (level == "NUTS3") {
      spei6_pw <- exactextractr::exact_extract(spei6_raster[[l]], adm1, "weighted_mean",
                                           weights = pop_era5land)
      spei12_pw <- exactextractr::exact_extract(spei12_raster[[l]], adm1, "weighted_mean",
                                           weights = pop_era5land)

      new_df <- data.frame(month = rep(substr(name, 6, 7), times = length(adm1_id)),
                           year = rep(substr(name, 1, 4), times = length(adm1_id)),
                           adm1_id = adm1_id,
                           #adm1_name = adm1_name,
                           spei6 = spei6,
                           spei12 = spei12,
                           spei6_pw = spei6_pw,
                           spei12_pw = spei12_pw)
    } else {
      spei6_pw <- NULL
      spei12_pw <- NULL
      new_df <- data.frame(month = rep(substr(name, 6, 7), times = length(adm1_id)),
                           year = rep(substr(name, 1, 4), times = length(adm1_id)),
                           adm1_id = adm1_id,
                           #adm1_name = adm1_name,
                           spei6 = spei6,
                           spei12 = spei12)
    }
    
    monthly_adm1 <- rbind(monthly_adm1, new_df)
    print(l)
  }

  write.csv(monthly_adm1, paste0("data/", level, "_spei_",
                                 paste0(accum, collapse = "-"),
                                 "_monthly_drought_indices.csv"),
            row.names = FALSE)
}
### END


