#!/usr/bin/env Rscript
## ---------------------------
##
## Script name: plot main figure
##
## Purpose of script: plot main figure
## Author: Daniela Lührsen & Emily Ball
## Date Created: 2025-03-07
## Email: daniela.luhrsen@bsc.es
##
## ---------------------------

library(sf)
library(ggplot2)
library(dplyr)
library(readr)
library(gridExtra)
library(cowplot)
library(patchwork)
library(rnaturalearth)
library(rnaturalearthdata)
library(RColorBrewer)
library(maps)


# select parameters:
level <- "NUTS3"
accum <- 12 # 6, 12
month <- 9
population_weight <- FALSE
# end parameter selection

region <- "europe"
weights <- c("_pw"[population_weight], ""[!population_weight])

# load shapefile
adm <- sf::st_read(paste0("../01_shapefiles/data/outputs/", level, ".shp"))
adm_id <- adm$adm1_id
adm_name <- adm$adm1_name

# transform to Eckert III to plot
eckert_crs <- "+proj=eck3 +datum=WGS84 +units=m +no_defs"
adm <- sf::st_transform(adm, crs = eckert_crs)

# remove ultraperipheric regions and islands
adm1 <- adm[which(
             adm$adm1_id != "FRY"   &
             adm$adm1_id != "FRY1"  &
             adm$adm1_id != "FRY2"  & 
             adm$adm1_id != "FRY3"  & 
             adm$adm1_id != "FRY4"  & 
             adm$adm1_id != "FRY5"  &
             adm$adm1_id != "FRY10" &
             adm$adm1_id != "FRY20" & 
             adm$adm1_id != "FRY30" & 
             adm$adm1_id != "FRY40" & 
             adm$adm1_id != "FRY50" &
             adm$adm1_id != "ES7"   &
             adm$adm1_id != "ES70"  &
             adm$adm1_id != "ES703" &
             adm$adm1_id != "ES704" &
             adm$adm1_id != "ES705" &
             adm$adm1_id != "ES706" &
             adm$adm1_id != "ES707" &
             adm$adm1_id != "ES708" &
             adm$adm1_id != "ES709" &
             adm$adm1_id != "PT2"   &
             adm$adm1_id != "PT20"  &
             adm$adm1_id != "PT200" &
             adm$adm1_id != "PT3"   &
             adm$adm1_id != "PT30"  &
             adm$adm1_id != "PT300" &
             adm$adm1_id != "NO0B"  &
             adm$adm1_id != "NO0B1" &
             adm$adm1_id != "NO0B2"
             ),]


# open SPEI file
data_csv <- read.csv(paste0("../03_aggregate_spatial/data/", level, "_",
                               "spei_6-12_monthly_drought_indices.csv"))

# select relevant SPEI accumulation and weighting
data_csv <- data_csv[, c("month", "year", "adm1_id", paste0("spei", accum, weights))]
names(data_csv)[which(names(data_csv) == paste0("spei", accum, weights))] <- "spei"

# define baseline and comparator periods
ref_years <- 1981:2010
comp_years <- 2015:2024

# select month to plot (month == 9 plots April-September SPEI6)
data_csv  <- data_csv[which(data_csv$month == month), ]
# count months below -1.6 (abnormally dry)
data_csv$spei <- data_csv$spei < -1.6
data_csv$spei[which(is.na(data_csv$spei))] <- FALSE

# separate to reference period and comparator period
ref_data  <- data_csv[which(data_csv$year %in% ref_years), ]
comp_data <- data_csv[which(data_csv$year %in% comp_years), ]

# percentage of years in each period in which SPEI = -1.6
ref_data  <- ref_data  %>% group_by(adm1_id) %>% summarise(spei_perc_ref  = 100 * mean(spei, na.rm = TRUE))
comp_data <- comp_data %>% group_by(adm1_id) %>% summarise(spei_perc_comp = 100 * mean(spei, na.rm = TRUE))

# combine reference and comparator
combined_df <- full_join(ref_data, comp_data, by = "adm1_id")

# merge shapefile
adm_merged <- merge(adm1, combined_df, by = "adm1_id")
adm_merged$spei_perc_diff <- adm_merged$spei_perc_comp - adm_merged$spei_perc_ref

############### plotting ###############
# Get land polygons and transform to same CRS
land <- ne_countries(scale = "medium", returnclass = "sf")
land <- sf::st_transform(land, sf::st_crs(adm_merged))
land <- sf::st_crop(land, adm_merged)

if (level == "country") {
  breaks <- c(0,1,2,3,4,5,6,7,8,9,10)
  lw <- 0.2
} else if (level == "NUTS3") {
  breaks <- c(0,4,8,12,16,20,24,28,32,36,40)
  lw <- 0.05
}

if (accum == 6) {
  if (level == "country") {
    breaks <- c(0,2,4,6,8,10,12,14,16,18,20)
    lw <- 0.2
  } else if (level == "NUTS3") {
    breaks <- c(0,4,8,12,16,20,24,28,32,36,40)
    lw <- 0.05
  }
}

reds_reversed <- brewer.pal(n = 9, name = "Reds")

# Left: percentage of years in reference period
p1 <- ggplot() +
  geom_sf(data = land, fill = "gray90", fill = NA) +  # Add land
  geom_sf(data = adm_merged, aes(fill = spei_perc_ref), color = "black", linewidth = lw) +
  scale_fill_stepsn(
                    breaks = breaks,
                    limits = range(breaks),
                    name='Percentage\nof years (%)', 
                    colours = reds_reversed,
                    guide = guide_colorbar(barheight = 12)
                    ) +
  theme_minimal() +
  geom_sf(data = land, fill = NA, color = "black", linewidth = 0.15) +  # Add borders
  ggtitle("(a) Percentage of years (1981-2010) experiencing extreme drought")

# Right: percentage of years in comparator period
p2 <- ggplot() +
  geom_sf(data = land, fill = "gray90", fill = NA) +  # Add land
  geom_sf(data = adm_merged, aes(fill = spei_perc_comp), color = "black", linewidth = lw) +
  scale_fill_stepsn(breaks = breaks,
                    name='Percentage\nof years (%)',
                    limits = range(breaks),
                    colours = reds_reversed,
                    guide = guide_colorbar(barheight = 12)
                    ) +
  theme_minimal() +
  geom_sf(data = land, fill = NA, color = "black", linewidth = 0.15) +  # Add borders
  ggtitle("(b)  Percentage of years (2015-2024) experiencing extreme drought")

# plot islands in inset panels
canarias <- adm[which(
             !(adm$adm1_id != "ES7"   &
               adm$adm1_id != "ES70"  &
               adm$adm1_id != "ES703" &
               adm$adm1_id != "ES704" &
               adm$adm1_id != "ES705" &
               adm$adm1_id != "ES706" &
               adm$adm1_id != "ES707" &
               adm$adm1_id != "ES708" &
               adm$adm1_id != "ES709")
             ),]

acores <- adm[which(
             !(adm$adm1_id != "PT2"   &
               adm$adm1_id != "PT20"  &
               adm$adm1_id != "PT200")
             ),]

madeira <- adm[which(
             !(adm$adm1_id != "PT3"   &
               adm$adm1_id != "PT30"  &
               adm$adm1_id != "PT300")
             ),]


# select spatial levels
if (level == "country") {
  canarias <- canarias %>% group_by(CNTR_CODE) %>% summarise(geometry = sf::st_union(geometry), .groups = "drop")
  acores   <- acores   %>% group_by(CNTR_CODE) %>% summarise(geometry = sf::st_union(geometry), .groups = "drop")
  madeira  <- madeira  %>% group_by(CNTR_CODE) %>% summarise(geometry = sf::st_union(geometry), .groups = "drop")
}

aco_poly <- sf::st_cast(acores, "POLYGON")
aco_poly <- aco_poly[-c(1),]
aco_poly <- aco_poly %>% summarise(geometry = sf::st_union(geometry), .groups = "drop")
acores$geometry <- aco_poly$geometry

mad_poly <- sf::st_cast(madeira, "POLYGON")
mad_poly <- mad_poly[-c(1,2),]
mad_poly <- mad_poly %>% summarise(geometry = sf::st_union(geometry), .groups = "drop")
madeira$geometry <- mad_poly$geometry


# merge shapefile and calculate percentage of events occurring in the comparator
canarias_merged <- merge(canarias, combined_df, by = "adm1_id")
acores_merged   <- merge(acores,   combined_df, by = "adm1_id")
madeira_merged  <- merge(madeira,  combined_df, by = "adm1_id")

canarias_merged$spei_perc_diff <- canarias_merged$spei_perc_comp - canarias_merged$spei_perc_ref
acores_merged$spei_perc_diff <- acores_merged$spei_perc_comp - acores_merged$spei_perc_ref
madeira_merged$spei_perc_diff <- madeira_merged$spei_perc_comp - madeira_merged$spei_perc_ref

land_can <- sf::st_crop(land, canarias_merged)
land_aco <- sf::st_crop(land, acores_merged)
land_mad <- sf::st_crop(land, madeira_merged)

land_aco <- sf::st_transform(land_aco, 5634)
acores_merged <- sf::st_transform(acores_merged, 5634)

land_mad <- sf::st_transform(land_mad, 5634)
madeira_merged <- sf::st_transform(madeira_merged, 5634)

land_can <- sf::st_transform(land_can, 5634)
canarias_merged <- sf::st_transform(canarias_merged, 5634)

lw <- 0.075

# Left: percentage of years in reference period
p_can1 <- ggplot() +
  geom_sf(data = land_can, fill = "gray90", color = NA) +  # Add land
  geom_sf(data = canarias_merged, aes(fill = spei_perc_ref), color = "black", linewidth = lw) +
  scale_fill_stepsn(breaks = breaks, 
                    limits = range(breaks),
                    colours = reds_reversed,
                    guide = guide_colorbar(barheight = 12)
                    ) +
  theme_void() +
  theme(
    panel.border = element_rect(fill = NA, colour = "black"),
    panel.background = element_rect(fill = alpha("lightblue", 0.35)),
    legend.position = "none",
    plot.title = element_text(size=7)
  ) +
  ggtitle("Canarias")

# Left: percentage of years in reference period
p_aco1 <- ggplot() +
  geom_sf(data = land_aco, fill = "gray90", color = NA) +  # Add land
  geom_sf(data = acores_merged, aes(fill = spei_perc_ref), color = "black", linewidth = lw) +
  scale_fill_stepsn(breaks = breaks, 
                    limits = range(breaks),
                    colours = reds_reversed,
                    guide = guide_colorbar(barheight = 12)
                    ) +
  theme_void() +
  theme(
    panel.border = element_rect(fill = NA, colour = "black"),
    panel.background = element_rect(fill = alpha("lightblue", 0.35)),
    legend.position = "none",
    plot.title = element_text(size=7)
  ) +
  ggtitle("Açores")

# Left: percentage of years in reference period
p_mad1 <- ggplot() +
  geom_sf(data = land_mad, fill = "gray90", color = NA) +  # Add land
  geom_sf(data = madeira_merged, aes(fill = spei_perc_ref), color = "black", linewidth = lw) +
  scale_fill_stepsn(breaks = breaks, 
                    limits = range(breaks),
                    colours = reds_reversed,
                    guide = guide_colorbar(barheight = 12)
                    ) +
  theme_void() +
  theme(
    panel.border = element_rect(fill = NA, colour = "black"),
    panel.background = element_rect(fill = alpha("lightblue", 0.35)),
    legend.position = "none",
    plot.title = element_text(size=7)
  ) +
  ggtitle("Madeira")

# add insets
p_out1 <- ggdraw() +
  draw_plot(p1) +
  draw_plot(p_can1, width = 0.07, x = 0.12, y = -0.275) +
  draw_plot(p_aco1, width = 0.07, x = 0.12, y = 0.025) +
  draw_plot(p_mad1, width = 0.07, x = 0.12, y = -0.15)

# Right: percentage of years in comparator period
p_can2 <- ggplot() +
  geom_sf(data = land_can, fill = "gray90", color = NA) +  # Add land
  geom_sf(data = canarias_merged, aes(fill = spei_perc_comp), color = "black", linewidth = lw) +
  scale_fill_stepsn(breaks = breaks, 
                    limits = range(breaks),
                    colours = reds_reversed,
                    guide = guide_colorbar(barheight = 12)
                    ) +
  theme_void() +
  theme(
    panel.border = element_rect(fill = NA, colour = "black"),
    panel.background = element_rect(fill = alpha("lightblue", 0.35)),
    legend.position = "none",
    plot.title = element_text(size=7)
  ) +
  ggtitle("Canarias")

# Right: percentage of years in comparator period
p_aco2 <- ggplot() +
  geom_sf(data = land_aco, fill = "gray90", color = NA) +  # Add land
  geom_sf(data = acores_merged, aes(fill = spei_perc_comp), color = "black", linewidth = lw) +
  scale_fill_stepsn(breaks = breaks, 
                    limits = range(breaks),
                    colours = reds_reversed,
                    guide = guide_colorbar(barheight = 12)
                    ) +
  theme_void() +
  theme(
    panel.border = element_rect(fill = NA, colour = "black"),
    panel.background = element_rect(fill = alpha("lightblue", 0.35)),
    legend.position = "none",
    plot.title = element_text(size=7)
  ) +
  ggtitle("Açores")

# Right: percentage of years in comparator period
p_mad2 <- ggplot() +
  geom_sf(data = land_mad, fill = "gray90", color = NA) +  # Add land
  geom_sf(data = madeira_merged, aes(fill = spei_perc_comp), color = "black", linewidth = lw) +
  scale_fill_stepsn(breaks = breaks, 
                    limits = range(breaks),
                    colours = reds_reversed,
                    guide = guide_colorbar(barheight = 12)
                    ) +
  theme_void() +
  theme(
    panel.border = element_rect(fill = NA, colour = "black"),
    panel.background = element_rect(fill = alpha("lightblue", 0.35)),
    legend.position = "none",
    plot.title = element_text(size=7)
  ) +
  ggtitle("Madeira")

p_out2 <- ggdraw() +
  draw_plot(p2) +
  draw_plot(p_can2, width = 0.07, x = 0.12, y = -0.275) +
  draw_plot(p_aco2, width = 0.07, x = 0.12, y = 0.025) +
  draw_plot(p_mad2, width = 0.07, x = 0.12, y = -0.15)

# Combine the two maps
final_plot <- p_out1 / p_out2

# Save the plot as a PNG file
ggsave(paste0("figures/figure_main_", region, "_", level, "_spei_", accum, "_map", weights, "_", month, ".png"),
       plot = final_plot, width = 7.25, height = 8, dpi = 300)

print(paste0("figures/figure_main_", region, "_", level, "_spei_", accum, "_map", weights, "_", month, ".png"))

# Save the plot as a pdf file
ggsave(paste0("figures/figure_main_", region, "_", level, "_spei_", accum, "_map", weights, "_", month, ".pdf"),
       plot = final_plot, width = 7.25, height = 8, dpi = 300)

print(paste0("figures/figure_main_", region, "_", level, "_spei_", accum, "_map", weights, "_", month, ".pdf"))




################# difference plot ###################
if (level == "country") {
  breaks <- c(-10,-7,-4,-1,1,4,7,10)
  lw <- 0.2
} else if (level == "NUTS3") {
  breaks <- c(-35,-25,-15,-5,5,15,25,35)
  lw <- 0.05
}
if (accum == 6) {
  if (level == "country") {
    breaks <- c(-20,-15,-10,-5,0,5,10,15,20)
    lw <- 0.2
  } else if (level == "NUTS3") {
    breaks <- c(-35,-25,-15,-5,5,15,25,35)
    lw <- 0.05
  }
}

reds_reversed <- rev(brewer.pal(n = 9, name = "RdBu"))

p_diff <- ggplot() +
  geom_sf(data = land, fill = "gray90", fill = NA) +  # Add land
  geom_sf(data = adm_merged, aes(fill = spei_perc_diff), color = "black", linewidth = lw) +
  scale_fill_stepsn(
                    breaks = breaks,
                    limits = range(breaks),
                    name='Change in\npercentage', 
                    colours = reds_reversed,
                    guide = guide_colorbar(barheight = 12)
                    ) +
  theme(
    panel.background = element_rect(fill = "white", color = NA),
    panel.grid.major = element_line(color = "gray90")
  ) +
  geom_sf(data = land, fill = NA, color = "black", linewidth = 0.15) +  # Add borders
  ggtitle("Change in percentage of years experiencing extreme drought")

# plot islands insets
p_can_diff <- ggplot() +
  geom_sf(data = land_can, fill = "gray90", color = NA) +  # Add land
  geom_sf(data = canarias_merged, aes(fill = spei_perc_diff), color = "black", linewidth = lw) +
  scale_fill_stepsn(breaks = breaks, 
                    limits = range(breaks),
                    colours = reds_reversed,
                    guide = guide_colorbar(barheight = 12)
                    ) +
  theme_void() +
  theme(
    panel.border = element_rect(fill = NA, colour = "black"),
    panel.background = element_rect(fill = alpha("lightblue", 0.35)),
    legend.position = "none",
    plot.title = element_text(size=7)
  ) +
  ggtitle("Canarias")

p_aco_diff <- ggplot() +
  geom_sf(data = land_aco, fill = "gray90", color = NA) +  # Add land
  geom_sf(data = acores_merged, aes(fill = spei_perc_diff), color = "black", linewidth = lw) +
  scale_fill_stepsn(breaks = breaks, 
                    limits = range(breaks),
                    colours = reds_reversed,
                    guide = guide_colorbar(barheight = 12)
                    ) +
  theme_void() +
  theme(
    panel.border = element_rect(fill = NA, colour = "black"),
    panel.background = element_rect(fill = alpha("lightblue", 0.35)),
    legend.position = "none",
    plot.title = element_text(size=7)
  ) +
  ggtitle("Açores")

p_mad_diff <- ggplot() +
  geom_sf(data = land_mad, fill = "gray90", color = NA) +  # Add land
  geom_sf(data = madeira_merged, aes(fill = spei_perc_diff), color = "black", linewidth = lw) +
  scale_fill_stepsn(breaks = breaks, 
                    limits = range(breaks),
                    colours = reds_reversed,
                    guide = guide_colorbar(barheight = 12)
                    ) +
  theme_void() +
  theme(
    panel.border = element_rect(fill = NA, colour = "black"),
    panel.background = element_rect(fill = alpha("lightblue", 0.35)),
    legend.position = "none",
    plot.title = element_text(size=7)
  ) +
  ggtitle("Madeira")

p_out_diff <- ggdraw() +
  draw_plot(p_diff) +
  draw_plot(p_can_diff,
    width = 0.07,
    x = 0.12,
    y = -0.275
  ) +
  draw_plot(p_aco_diff,
    width = 0.07,
    x = 0.12,
    y = 0.025
  ) +
  draw_plot(p_mad_diff,
    width = 0.07,
    x = 0.12,
    y = -0.15
  )

final_plot <- p_out_diff

# Save the plot as a PNG file
ggsave(paste0("figures/figure_diff_", region, "_", level, "_spei_", accum, "_map", weights, "_", month, ".png"),
       plot = final_plot, width = 7.25, height = 4, dpi = 300)

print(paste0("figures/figure_diff_", region, "_", level, "_spei_", accum, "_map", weights, "_", month, ".png"))

# Save the plot as a pdf file
ggsave(paste0("figures/figure_diff_", region, "_", level, "_spei_", accum, "_map", weights, "_", month, ".pdf"),
       plot = final_plot, width = 7.25, height = 4, dpi = 300)

print(paste0("figures/figure_diff_", region, "_", level, "_spei_", accum, "_map", weights, "_", month, ".pdf"))
