#!/usr/bin/env Rscript
## ---------------------------
##
## Script name: plot timeseries SPEI, PET and precipitation
##
## Purpose of script: plot timeseries SPEI, PET and precipitation
## Author: Daniela Lührsen & Emily Ball
## Date Created: 2025-03-07
## Email: daniela.luhrsen@bsc.es
##
## ---------------------------

# load packages
packages <- c("raster", "dplyr", "tidyr", "ggplot2", "stringr", "terra", "patchwork")
lapply(packages, require, character.only = TRUE)

# select parameters
accum <- 6
region <- "europe"
month <- 3

# load the masked data
spei_masked <- unwrap(readRDS(paste0("../02_calculate_spei/data/temp/europe-masked_spei_", accum, ".rds")))

# extract spei_masked layers to dates
layer_names <- names(spei_masked)
time_full <- str_extract(layer_names, "\\d{4}\\.\\d{2}.\\d{2}")
time_full <- str_replace(time_full, "\\.", "-")
time_full <- str_replace(time_full, "\\.", "-")
names(spei_masked) <- as.Date(time_full)

# define a function to classify SPEI into categories
classify_spei <- function(x) {
  ifelse(is.na(x), NA,
         ifelse(x <= -2, "Exceptional drought",
                ifelse(x <= -1.6, "Extreme drought", 
                       ifelse(x <= -1.3, "Severe drought",
                              ifelse(x <= -0.8, "Moderate drought",
                                     ifelse(x <= -0.5, "Abnormally dry", NA))))))
}

# initialize dataframe to store results
spei_stats <- data.frame()

# Loop through each time step (month)
for (i in 1:nlyr(spei_masked)) {
  if (lubridate::month(names(spei_masked)[i]) == month) {
    cat("Processing layer:", names(spei_masked)[i], "\n")
  
    # Get raster layer
    layer <- spei_masked[[i]]
  
    # Extract values as vector
    values <- terra::values(layer)
  
    # Classify values
    class_vals <- classify_spei(values)
  
    # Calculate proportions
    # sum(!is.na(values)) - total land area
    prop_table <- table(class_vals, useNA = "no") / sum(!is.na(values))

    if (length(prop_table) != 0){
      # Store results
      temp_df <- as.data.frame(prop_table) %>%
        rename(category = class_vals, proportion = Freq) %>%
        mutate(month = lubridate::month(names(spei_masked)[i]),
               year  = lubridate::year(names(spei_masked)[i]))
    
      spei_stats <- bind_rows(spei_stats, temp_df)    
    }    
  }
}

# View result
print(head(spei_stats))
spei_stats$date <- as.Date(paste0(spei_stats$year,"-", spei_stats$month,"-01"))
drought_colors <- c(
  "Abnormally dry"      = "#ffff44",  # light yellow
  "Moderate drought"    = "#ffd487",  # beige-orange
  "Severe drought"      = "#ffab32",  # orange
  "Extreme drought"     = "#f1061b",  # red
  "Exceptional drought" = "#790108",  # dark red
  "bla"                 = "grey90"    # for NA or uncategorized
)
spei_stats$category <- factor(spei_stats$category, levels = c(
  "", "Abnormally dry", "Moderate drought", "Severe drought",
  "Extreme drought", "Exceptional drought"
))
# plot time series
plot_out <- ggplot(spei_stats, aes(x = lubridate::year(date), y = proportion, fill = category)) +
  geom_area(position = "stack", color = NA) +
  scale_fill_manual(values = drought_colors) +
  scale_y_continuous(labels = scales::percent_format(accuracy = 1))+#,
                     #sec.axis = sec_axis(~ ., name = "", labels = NULL)) +
  theme_minimal(base_size = 14) +
  theme(
    panel.grid.minor = element_blank(),
    # panel.grid.major.x = element_blank(),
    legend.position = "bottom",
    legend.title = element_blank()
  ) +
  labs(
    # title = "(a)",
    x = NULL,
    y = "Percentage land area affected"
  )


# Save the plot as a PNG file
ggsave(paste0("figures/drought_class_", region, "_spei_", accum, "_", month, ".png"),
       plot = plot_out, width = 12, height = 4, dpi = 300)

print(paste0("figures/drought_class_", region, "_spei_", accum, "_", month, ".png"))

ggsave(paste0("figures/drought_class_", region, "_spei_", accum, "_", month, ".pdf"),
       plot = plot_out, width = 12, height = 4, dpi = 300)

print(paste0("figures/drought_class_", region, "_spei_", accum, "_", month, ".pdf"))


### combine panels
source("figure_03.R")

final_plot <- ( plot_out / p_1 )  + plot_layout(heights = c(4,3)) + plot_annotation(tag_levels = "a",
                                                   tag_prefix = "(",
                                                   tag_suffix = ")")

# Save the plot as a PNG file
ggsave(paste0("figures/timeseries_", region, "_spei_", accum, "_", month, ".png"),
       plot = final_plot, width = 12, height = 7, dpi = 300)

print(paste0("figures/timeseries_", region, "_spei_", accum, "_", month, ".png"))

ggsave(paste0("figures/timeseries_", region, "_spei_", accum, "_", month, ".pdf"),
       plot = final_plot, width = 12, height = 7, dpi = 300)

print(paste0("figures/timeseries_", region, "_spei_", accum, "_", month, ".pdf"))
