#!/usr/bin/env Rscript
## ---------------------------
##
## Script name: plot regions/population exposed to drought
##
## Purpose of script: plot percentage of population/regions and calculate
## changes for reference and comparator periods.
##
## Author: Daniela Lührsen & Emily Ball
## Date Created: 2025-03-07
## Email: daniela.luhrsen@bsc.es
##
## ---------------------------

# load packages
library(terra)
library(sf)
library(dplyr)
library(readr)
library(data.table)
library(ggplot2)

# select parameters:
region <- "europe"
level <- "NUTS3"
population_weight <- FALSE
accum <- 12 # 6, 12
month <- 3

# define baseline and comparator periods
ref_years <- 1981:2010
comp_years <- 2015:2024

weights <- c("_pw"[population_weight], ""[!population_weight])

# open SPEI file
data_csv <- read.csv(paste0("../03_aggregate_spatial/data/", level, "_spei_6-12_monthly_drought_indices.csv"))

data_csv <- data_csv[, c("month", "year", "adm1_id", paste0("spei", accum, weights))]
names(data_csv)[which(names(data_csv) == paste0("spei", accum, weights))] <- "spei"

data_csv  <- data_csv[which(data_csv$month == month), ]

# select years in the comparator period
comp_data <- data_csv[which(data_csv$year %in% comp_years), ]
comp_data$spei <- comp_data$spei < -1.6
comp_data$spei[which(is.na(comp_data$spei))] <- FALSE
comp_all <- comp_data %>% group_by(adm1_id) %>% summarise(spei_sum_comp = any(spei, na.rm = TRUE))

#### percentage of regions experiencing at least one year of extreme-exceptional drought in 2015-2024
print(100*mean(comp_all$spei_sum_comp))

#######################################################################

# select years in reference period
ref_data <- data_csv[which(data_csv$year %in% ref_years), ]
ref_data$spei <- ref_data$spei < -1.6
ref_data$spei[which(is.na(ref_data$spei))] <- FALSE
ref_all <- ref_data %>% group_by(adm1_id) %>% summarise(spei_sum_ref = 100*mean(spei, na.rm = TRUE))

comp_data <- data_csv[which(data_csv$year %in% comp_years), ]
comp_data$spei <- comp_data$spei < -1.6
comp_data$spei[which(is.na(comp_data$spei))] <- FALSE
comp_all <- comp_data %>% group_by(adm1_id) %>% summarise(spei_sum_comp = 100*mean(spei, na.rm = TRUE))

combined_df <- full_join(ref_all, comp_all, by = "adm1_id")
combined_df$spei_diff <- combined_df$spei_sum_comp - combined_df$spei_sum_ref


#### increase in the percentage of years experiencing extreme-exceptional drought
combined_df$increase <- combined_df$spei_diff > 0
print(mean(combined_df$increase)*100)

#######################################################################################
# comparing 1981-2024 and 2015-2024

# define baseline and comparator periods
ref_years <- 1981:2024
comp_years <- 2015:2024


ref_data <- data_csv[which(data_csv$year %in% ref_years), ]
ref_data$spei <- ref_data$spei < -1.6
ref_data$spei[which(is.na(ref_data$spei))] <- FALSE

comp_data <- ref_data[which(ref_data$year %in% comp_years), ]
ref_all  <-  ref_data %>% group_by(adm1_id) %>% summarise(spei_sum_ref = sum(spei, na.rm = TRUE))
comp_all <- comp_data %>% group_by(adm1_id) %>% summarise(spei_sum_comp = sum(spei, na.rm = TRUE))


combined_df <- full_join(ref_all, comp_all, by = "adm1_id")
combined_df$spei_perc <- combined_df$spei_sum_comp / combined_df$spei_sum_ref * 100

combined_df$spei_perc[which(is.na(combined_df$spei_perc))] <- 0

combined_df$perc_30 <- combined_df$spei_perc > 30

### percentage of regions where at least 30% of drought events occurred in comparator
print(mean(combined_df$perc_30)* 100)



######## plot figure #########


# open SPEI file
data_csv <- read.csv(paste0("../03_aggregate_spatial/data/", level, "_spei_6-12_monthly_drought_indices.csv"))

data_csv  <- data_csv[which(data_csv$month == month), ]
data_csv  <- data_csv[which(data_csv$year > 1980), ]

data_csv$spei6_pw  <- data_csv$spei6_pw  < -1.6
data_csv$spei12_pw <- data_csv$spei12_pw < -1.6
data_csv$spei6     <- data_csv$spei6     < -1.6
data_csv$spei12    <- data_csv$spei12    < -1.6

data_csv_tmp <- data_csv %>%
  group_by(year, month) %>% summarise(spei6     = 100 * mean(spei6,     na.rm = TRUE),
                                      spei12    = 100 * mean(spei12,    na.rm = TRUE),
                                      spei6_pw  = 100 * mean(spei6_pw,  na.rm = TRUE),
                                      spei12_pw = 100 * mean(spei12_pw, na.rm = TRUE),)


df <- data.frame(
  Date      = data_csv_tmp$year,
  SPEI6     = data_csv_tmp$spei6,
  SPEI12    = data_csv_tmp$spei12,
  SPEI6_pw  = data_csv_tmp$spei6_pw,
  SPEI12_pw = data_csv_tmp$spei12_pw
)

spei6  <- data_csv_tmp$spei6 / 100
spei12 <- data_csv_tmp$spei12 / 100

df_long <- melt(df, id.vars = "Date", variable.name = "Variable", value.name = "Value")

p_1 <- ggplot(df_long, aes(x = Date, y = Value, color = Variable, linetype = Variable)) +
  geom_line(linewidth = 0.8) +
  scale_color_manual(
    values = c("SPEI6" = "blue", "SPEI12" = "red",
               "SPEI6_pw" = "blue", "SPEI12_pw" = "red"),
    labels = c("SPEI6", "SPEI12", "SPEI6 weighted", "SPEI12 weighted")
  ) +
  scale_linetype_manual(
    values = c("SPEI6" = "solid", "SPEI12" = "solid",
               "SPEI6_pw" = "dashed", "SPEI12_pw" = "dashed"),
    labels = c("SPEI6", "SPEI12", "SPEI6 weighted", "SPEI12 weighted")
  ) +
  scale_y_continuous(
    name = "Percentage of regions experiencing\nextreme to exceptional drought"
  ) +
  labs(x = NULL, y = NULL) +
  theme_minimal(base_size = 14) +
  theme(
    legend.position = c(0.8,0.9),
    legend.background = element_rect(fill = alpha("white", 0.7), color = NA),
    legend.title = element_blank(),
    panel.grid.minor = element_blank(),
    plot.title = element_blank()
  )

final_plot <- p_1


# open population density (pop per km2)
pop_density <- readRDS(paste0("../02_calculate_spei/data/", region,
                               "_population_stack.rds"))
cell_areas <- terra::area(pop_density)
pop_era5land <- pop_density * cell_areas
adm1 <- sf::st_read("../01_shapefiles/data/outputs/NUTS3.shp")

# extract to NUTS3 level
pop_shp <- exactextractr::exact_extract(pop_era5land, adm1, "sum")
eur_pop <- sum(pop_shp)
pop_shp <- rep(pop_shp, max(data_csv$year) - min(data_csv$year) + 1)

data_csv_tmp <- cbind(data_csv, pop_shp)

data_csv_tmp$spei6_exp     <- data_csv_tmp$spei6     * data_csv_tmp$pop_shp
data_csv_tmp$spei12_exp    <- data_csv_tmp$spei12    * data_csv_tmp$pop_shp
data_csv_tmp$spei6_pw_exp  <- data_csv_tmp$spei6_pw  * data_csv_tmp$pop_shp
data_csv_tmp$spei12_pw_exp <- data_csv_tmp$spei12_pw * data_csv_tmp$pop_shp

data_csv_tmp <- data_csv_tmp %>%
  group_by(year, month) %>% summarise(    spei6 = sum(    spei6_exp, na.rm = TRUE)/eur_pop,
                                         spei12 = sum(   spei12_exp, na.rm = TRUE)/eur_pop,
                                       spei6_pw = sum( spei6_pw_exp, na.rm = TRUE)/eur_pop,
                                      spei12_pw = sum(spei12_pw_exp, na.rm = TRUE)/eur_pop,)



df <- data.frame(
  Date       = data_csv_tmp$year,
  SPEI6_pop  = data_csv_tmp$spei6,
  SPEI12_pop = data_csv_tmp$spei12,
  SPEI6_reg  = spei6,
  SPEI12_reg = spei12
)


df_long <- melt(df, id.vars = "Date", variable.name = "Variable", value.name = "Value")

p_1 <- ggplot(df_long, aes(x = Date)) +
  geom_line(
    data = filter(df_long, Variable %in% c("SPEI6_pop", "SPEI12_pop")),
    aes(y = Value, color = Variable, linetype = Variable),
    linewidth = 0.8
  ) +
  geom_line(
    data = filter(df_long, Variable %in% c("SPEI6_reg", "SPEI12_reg")),
    aes(y = Value, color = Variable, linetype = Variable),
    linewidth = 0.8
  ) +
  scale_color_manual(
    values = c("SPEI6_pop" = "blue", "SPEI12_pop" = "red",
               "SPEI6_reg" = "blue", "SPEI12_reg" = "red"),
    labels = c("SPEI6 (by population)", "SPEI12 (by population)",
               "SPEI6 (by region)", "SPEI12 (by region)")
  ) +
  scale_linetype_manual(
    values = c("SPEI6_pop" = "solid", "SPEI12_pop" = "solid",
               "SPEI6_reg" = "dashed", "SPEI12_reg" = "dashed"),
    labels = c("SPEI6 (by population)", "SPEI12 (by population)",
               "SPEI6 (by region)", "SPEI12 (by region)")
  ) +
  scale_y_continuous(
    name = "Percentage of regions experiencing\nextreme to exceptional drought",
    labels = scales::percent_format(accuracy = 1),
    sec.axis = sec_axis(~ ., name = "Percentage of population experiencing\nextreme to exceptional drought")
  ) +
  labs(x = NULL, y = NULL) +
  theme_minimal(base_size = 14) +
  theme(
    legend.position = c(0.7,0.85),
    legend.background = element_rect(fill = alpha("white", 0.7), color = NA),
    legend.title = element_blank(),
    panel.grid.minor = element_blank(),
    plot.title = element_blank()
  )

final_plot <- p_1

# Save the plot as a PNG file
ggsave(paste0("figures/percentage_pop_", region, "_", month, ".png"),
       plot = final_plot, width = 12, height = 4, dpi = 300)

print(paste0("figures/percentage_pop_", region, "_", month, ".png"))

ggsave(paste0("figures/percentage_pop_", region, "_", month, ".pdf"),
       plot = final_plot, width = 12, height = 4, dpi = 300)

print(paste0("figures/percentage_pop_", region, "_", month, ".pdf"))

